from django.db import models

from django.contrib.contenttypes.models import ContentType

from agon_ratings.categories import category_value


class OverallRatingManager(models.Manager):
    
    def top_rated(self, klass, category=None):
        
        if category:
            cat = category_value(klass, category)
        else:
            cat = None
        
        return self.filter(
            content_type=ContentType.objects.get_for_model(klass),
            category=cat
        ).extra(
            select={
                "sortable_rating": "COALESCE(rating, 0)"
            }
        ).order_by("-sortable_rating")



try:
    from django.conf.urls import url, patterns
except ImportError:
    from django.conf.urls.defaults import url, patterns


urlpatterns = patterns("agon_ratings.views",
    url(r"^(?P<content_type_id>\d+)/(?P<object_id>\d+)/rate/$", "rate", name="agon_ratings_rate"),
)



from django.conf import settings


RATING_CATEGORY_CHOICES_DICT = getattr(settings, "AGON_RATINGS_CATEGORY_CHOICES", {})
RATING_CATEGORY_CHOICES = []
RATING_CATEGORY_LOOKUP = {}
if len(RATING_CATEGORY_CHOICES_DICT.keys()) > 0:
    for model_str in RATING_CATEGORY_CHOICES_DICT.keys():
        for choice in RATING_CATEGORY_CHOICES_DICT[model_str].keys():
            slug = "%s-%s" % (model_str, choice)
            val = len(RATING_CATEGORY_CHOICES) + 1
            RATING_CATEGORY_CHOICES.append((val, slug))
            RATING_CATEGORY_LOOKUP[slug] = val


def category_label(obj, choice):
    obj_str = "%s.%s" % (obj._meta.app_label, obj._meta.object_name)
    return RATING_CATEGORY_CHOICES_DICT.get(obj_str, {}).get(choice)


def is_valid_category(obj, choice):
    return category_label(obj, choice) is not None


def category_value(obj, choice):
    return RATING_CATEGORY_LOOKUP.get(
        "%s.%s-%s" % (obj._meta.app_label, obj._meta.object_name, choice)
    )



import datetime

from decimal import Decimal

from django.db import models

from django.conf import settings

from django.contrib.auth.models import User
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from agon_ratings.categories import RATING_CATEGORY_CHOICES
from agon_ratings.managers import OverallRatingManager

USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', User)

class OverallRating(models.Model):
    
    object_id = models.IntegerField(db_index=True)
    content_type = models.ForeignKey(ContentType)
    content_object = GenericForeignKey()
    rating = models.DecimalField(decimal_places=1, max_digits=6, null=True)
    
    category = models.IntegerField(null=True, choices=RATING_CATEGORY_CHOICES)
    
    objects = OverallRatingManager()
    
    class Meta:
        unique_together = [
            ("object_id", "content_type", "category"),
        ]
    
    def update(self):
        self.rating = Rating.objects.filter(
            overall_rating = self
        ).aggregate(r = models.Avg("rating"))["r"]
        self.rating = Decimal(str(self.rating or "0"))
        self.save()


class Rating(models.Model):
    overall_rating = models.ForeignKey(OverallRating, null = True, related_name = "ratings")
    object_id = models.IntegerField(db_index=True)
    content_type = models.ForeignKey(ContentType)
    content_object = GenericForeignKey()
    user = models.ForeignKey(USER_MODEL)
    rating = models.IntegerField()
    timestamp = models.DateTimeField(default=datetime.datetime.now)
    
    category = models.IntegerField(null=True, choices=RATING_CATEGORY_CHOICES)
    
    class Meta:
        unique_together = [
            ("object_id", "content_type", "user", "category"),
        ]
    
    def __unicode__(self):
        return unicode(self.rating)



from django.conf import settings
from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import get_object_or_404
try:
    import json
except ImportError:
    from django.utils import simplejson as json
from django.views.decorators.http import require_POST

from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType

from agon_ratings.categories import category_value
from agon_ratings.models import Rating, OverallRating


NUM_OF_RATINGS = getattr(settings, "AGON_NUM_OF_RATINGS", 5)


@require_POST
@login_required
def rate(request, content_type_id, object_id):
    ct = get_object_or_404(ContentType, pk=content_type_id)
    obj = get_object_or_404(ct.model_class(), pk=object_id)
    rating_input = int(request.POST.get("rating"))
    category = request.POST.get("category")
    if category:
        cat_choice = category_value(obj, category)
    else:
        cat_choice = None
    
    # Check for errors and bail early
    if category is not None and cat_choice is None:
        return HttpResponseForbidden(
            "Invalid category. It must match a preconfigured setting"
        )
    if rating_input not in range(NUM_OF_RATINGS + 1):
        return HttpResponseForbidden(
            "Invalid rating. It must be a value between 0 and %s" % NUM_OF_RATINGS
        )
    
    data = {
        "user_rating": rating_input,
        "overall_rating": 0,
        "category": category
    }
    
    # @@@ Seems like this could be much more DRY with a model method or something
    if rating_input == 0: # clear the rating
        try:
            rating = Rating.objects.get(
                object_id = object_id,
                content_type = ct,
                user = request.user,
                category = cat_choice
            )
            overall = rating.overall_rating
            rating.delete()
            overall.update()
            data["overall_rating"] = str(overall.rating)
        except Rating.DoesNotExist:
            pass
    else: # set the rating
        rating, created = Rating.objects.get_or_create(
            object_id = obj.pk,
            content_type = ct,
            user = request.user,
            category = cat_choice,
            defaults = {
                "rating": rating_input
            }
        )
        overall, created = OverallRating.objects.get_or_create(
            object_id = obj.pk,
            content_type = ct,
            category = cat_choice
        )
        rating.overall_rating = overall
        rating.rating = rating_input
        rating.save()
        overall.update()
        data["overall_rating"] = str(overall.rating)
    
    return HttpResponse(json.dumps(data), mimetype="application/json")



from decimal import Decimal

from django import template
from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse

from django.contrib.contenttypes.models import ContentType

from agon_ratings.categories import category_value
from agon_ratings.models import Rating, OverallRating


register = template.Library()


def user_rating_value(user, obj, category=None):
    try:
        ct = ContentType.objects.get_for_model(obj)
        if category is None:
            rating = Rating.objects.filter(
                object_id = obj.pk,
                content_type = ct,
                user = user
            ).aggregate(r = models.Avg("rating"))["r"]
            rating = Decimal(str(rating or "0"))
        else:
            rating = Rating.objects.get(
                object_id = obj.pk,
                content_type = ct,
                user = user,
                category = category_value(obj, category)
            ).rating
    except Rating.DoesNotExist:
        rating = 0
    return rating


class UserRatingNode(template.Node):
    
    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        
        if len(bits) == 5:
            category = None
        elif len(bits) == 6:
            category = parser.compile_filter(bits[3])
        else:
            raise template.TemplateSyntaxError()
        
        return cls(
            user = parser.compile_filter(bits[1]),
            obj = parser.compile_filter(bits[2]),
            as_var = bits[len(bits) - 1],
            category = category
        )
    
    def __init__(self, user, obj, as_var, category=None):
        self.user = user
        self.obj = obj
        self.as_var = as_var
        self.category = category
    
    def render(self, context):
        user = self.user.resolve(context)
        obj = self.obj.resolve(context)
        if self.category:
            category = self.category.resolve(context)
        else:
            category = None
        context[self.as_var] = user_rating_value(user, obj, category)
        return ""


@register.tag
def user_rating(parser, token):
    """
    Usage:
        {% user_rating user obj [category] as var %}
    """
    return UserRatingNode.handle_token(parser, token)


class OverallRatingNode(template.Node):
    
    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        
        if len(bits) == 4:
            category = None
        elif len(bits) == 5:
            category = parser.compile_filter(bits[2])
        else:
            raise template.TemplateSyntaxError()
        
        return cls(
            obj = parser.compile_filter(bits[1]),
            as_var = bits[len(bits) - 1],
            category = category
        )
    
    def __init__(self, obj, as_var, category=None):
        self.obj = obj
        self.as_var = as_var
        self.category = category
    
    def render(self, context):
        obj = self.obj.resolve(context)
        if self.category:
            category = self.category.resolve(context)
        else:
            category = None
        
        try:
            ct = ContentType.objects.get_for_model(obj)
            if category is None:
                rating = OverallRating.objects.filter(
                    object_id = obj.pk,
                    content_type = ct
                ).aggregate(r = models.Avg("rating"))["r"]
                rating = Decimal(str(rating or "0"))
            else:
                rating = OverallRating.objects.get(
                    object_id = obj.pk,
                    content_type = ct,
                    category = category_value(obj, category)
                ).rating or 0
        except OverallRating.DoesNotExist:
            rating = 0
        context[self.as_var] = rating
        return ""


@register.tag
def overall_rating(parser, token):
    """
    Usage:
        {% overall_rating obj [category] as var %}
    """
    return OverallRatingNode.handle_token(parser, token)


def rating_post_url(user, obj):
    ct = ContentType.objects.get_for_model(obj)
    post_url = reverse(
        "agon_ratings_rate",
        kwargs = {
            "content_type_id": ct.pk,
            "object_id": obj.pk
        }
    )
    return post_url


@register.inclusion_tag("agon_ratings/_script.html")
def user_rating_js(user, obj, category=None):
    post_url = rating_post_url(user, obj)
    rating = user_rating_value(user, obj, category)
    
    return {
        "obj": obj,
        "post_url": post_url,
        "category": category,
        "the_user_rating": rating,
        "STATIC_URL": settings.STATIC_URL,
    }


@register.assignment_tag
def ratings(obj):
    ct = ContentType.objects.get_for_model(obj)
    try:
        return OverallRating.objects.get(
            content_type=ct,
            object_id=obj.pk,
            category=None
        ).ratings.all()
    except OverallRating.DoesNotExist:
        return []


@register.simple_tag
def user_rating_url(user, obj):
    return rating_post_url(user, obj)

